﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace ClockgeneratorControl
{
	public delegate void ChangedEventHandler(object sender, EventArgs e);
	
	public partial class ClockgeneratorControl : UserControl
	{
		#region Vars
		public event ChangedEventHandler Changed;
		private int TrackBarValue=0;
		private bool AktivCalc=false;
		
		#endregion

		#region Events and Constructor
		public ClockgeneratorControl(){
			InitializeComponent();
			actionsTimer.Interval=SpeedTrackBar.Value;
			SelectStatus.Items.Add("Off");
			SelectStatus.Items.Add("Binär");
			SelectStatus.Items.Add("BCD");
			SelectStatus.Items.Add("Gray");
			SelectStatus.SelectedIndex=0;
			CheckBoxTimerCheckedChanged(null,EventArgs.Empty);
		}
		
		private void SelectStatusSelectedItemChanged(object sender, EventArgs e){
			SetState(true);
			ValueChanged();
		}
		
		private void SetState(bool EnableIt){
			if (SelectStatus.SelectedItem.ToString()=="Off"){
				TextBox.Enabled=false;
				actionsTimer.Stop();
				SpeedTrackBar.Enabled=false;
				ValueTrackBar.Enabled=false;
				CheckBoxTimer.Enabled=false;
				bnStep.Enabled=false;
				textLSB.Enabled=false;
				textMSB.Enabled=false;
			}else{
				TextBox.Enabled=true;
				ValueTrackBar.Enabled=true;
				CheckBoxTimer.Enabled=true;
				CheckBoxTimerCheckedChanged(null,EventArgs.Empty);
				textLSB.Enabled=EnableIt;
				textMSB.Enabled=EnableIt;
			}
		}

		private void SpeedTrackBarScroll(object sender, EventArgs e){
			actionsTimer.Interval=SpeedTrackBar.Value;
		}
		
		private void ValueTrackBarScroll(object sender, EventArgs e){
			ValueChanged();
		}
		
		private void CheckBoxTimerCheckedChanged(object sender, EventArgs e){
			if ((CheckBoxTimer.Checked)&&(SelectStatus.SelectedItem.ToString()!="Off")){
				actionsTimer.Start();
				SpeedTrackBar.Enabled=true;
			}else {
				actionsTimer.Stop();
				SpeedTrackBar.Enabled=false;
			}
			bnStep.Enabled=!CheckBoxTimer.Checked;
			CalcBorder();
		}
		
		private void BnStepClick(object sender, EventArgs e){
			actionsTimer_Tick(null,EventArgs.Empty);
		}
		#endregion

		#region Methods
		
		private void actionsTimer_Tick(object sender, EventArgs e){
			if (ValueTrackBar.Maximum==ValueTrackBar.Value) ValueTrackBar.Value=0;
			else ValueTrackBar.Value++;
			ValueChanged();
		}
		
		public int ConvertDezimal2BCD(int Val){
			int Value=0;
			string strVal=Val.ToString();
			for(int i=strVal.Length-1;i>=0;i--){
				int Ziffer=int.Parse(strVal[i].ToString());
				Ziffer*=(int)Math.Pow(16,strVal.Length-i-1);
				Value+=Ziffer;
			}
			return Value;
		}
		
		public void ValueChanged(){
			if(AktivCalc) return;
			
			int MaxD;int MaxB;int MaxH;
			try{
				int Bitanzahl=(int)Math.Abs(int.Parse(textMSB.Text)-int.Parse(textLSB.Text))+1;
				MaxD=(int)Math.Ceiling(Math.Log10(2)*Bitanzahl);
				MaxB=Bitanzahl;
				MaxH=(int)Math.Ceiling(Bitanzahl/4.0);
			}catch(Exception){
				MaxD=(int)Math.Ceiling(Math.Log10(ValueTrackBar.Maximum));
				MaxB=(int)Math.Ceiling(Math.Log(ValueTrackBar.Maximum,2));
				MaxH=(int)Math.Ceiling(Math.Log(ValueTrackBar.Maximum,16));
			}
			
			if (SelectStatus.SelectedItem.ToString()=="BCD"){
				TrackBarValue=ConvertDezimal2BCD(ValueTrackBar.Value);
			}
			else if(SelectStatus.SelectedItem.ToString()=="Gray"){
				TrackBarValue=(ValueTrackBar.Value>>1) ^ ValueTrackBar.Value;
			}else
				TrackBarValue=ValueTrackBar.Value;
			
			TextBox.Text=
				InsertPoints(TrackBarValue.ToString("D"),MaxD)+"d "+
				InsertPoints(TrackBarValue.ToString("X"),MaxH)+"h  "+
				InsertPoints(Convert.ToString(TrackBarValue, 2),MaxB)+"b      "+
				ValueTrackBar.Value.ToString("D5");
			TextBox.SelectAll();
			TextBox.SelectionAlignment= HorizontalAlignment.Left;
			TextBox.SelectionLength=0;
			if (Changed != null) Changed(this, EventArgs.Empty);
		}
		
		public string InsertPoints(string Val,int Size){
			string Out="";
			if (Size>Val.Length) Val=new String('0',Size-Val.Length)+Val;
			for(int i=0;i<Val.Length;i++){
				if((i%4==0)&&(i!=0)){
					Out="."+Out;
				}
				Out=Val[Val.Length-i-1]+Out;
			}
			return Out;
		}
		
		public string insertString(string Input){
			if (SelectStatus.SelectedItem.ToString()=="Off")
				return Input;
			else{
				int Start;
				int Length;
				try{
					Start=Math.Min(int.Parse(textMSB.Text),int.Parse(textLSB.Text));
					Length=Math.Abs(int.Parse(textMSB.Text)-int.Parse(textLSB.Text))+1;
				}catch(Exception){
					return Input;
				}
				if(Input.Length<(Start+Length)){
					if(Input.Length<Start) Start=Input.Length-1;
					Length=Input.Length-Start;
				}
				string Text=Convert.ToString(TrackBarValue, 2);
				if(Text.Length<Length) Text=new String('0',Length-Text.Length)+Text;
				else Text=Text.Substring(0,Length);
				char [] Reverse=Text.ToCharArray();
				Array.Reverse(Reverse);

				Input=Input.Remove(Start,Length);
				return Input.Insert(Start,new string(Reverse));
			}
		}
		#endregion
		
		private void ChangeText(TextBox box,int Value){
			int start=box.SelectionStart;
			box.Text=Value.ToString();
			box.SelectionStart=(int)Math.Min(start,box.Text.Length);
		}
		
		private void CalcBorder(){
			if(AktivCalc) return;
			AktivCalc=true;
			actionsTimer.Stop();
			int MSB;
			int LSB;
			try{
				MSB=int.Parse(textMSB.Text);
				LSB=int.Parse(textLSB.Text);
			}catch(Exception){
				AktivCalc=false;
				return;
			}
			int Bitanzahl=Math.Abs(MSB-LSB)+1;
			if(Bitanzahl>24){
				int diff=Bitanzahl-24;
				if(MSB>LSB) ChangeText(textMSB,MSB-diff);
				else ChangeText(textLSB,LSB-diff);
				Bitanzahl=24;
			}
			if(MSB>31) ChangeText(textMSB,31);
			if(LSB>31) ChangeText(textLSB,31);
			
			
			int Wertebereich=(int)Math.Round(Math.Pow(2,Bitanzahl));
			
			if (SelectStatus.SelectedItem.ToString()=="BCD"){
				string Out="";
				foreach (char hexValue in Wertebereich.ToString("X")){
					int dez=Int32.Parse(hexValue.ToString(), System.Globalization.NumberStyles.HexNumber);
					 if(dez>9) dez=9;
					 Out=Out+dez.ToString();
				}
				Wertebereich=int.Parse(Out);
			}
			ValueTrackBar.Hide();
			ValueTrackBar.Maximum=Wertebereich-1;
			ValueTrackBar.TickFrequency=(int)Math.Ceiling(Wertebereich/16.0);
			ValueTrackBar.LargeChange=(int)Math.Ceiling(Wertebereich/64.0);
			ValueTrackBar.Show();
			CheckBoxTimerCheckedChanged(null,EventArgs.Empty);
			AktivCalc=false;
		}

		void TextMSBKeyPress(object sender, KeyPressEventArgs e){
			base.OnKeyPress(e);
			string keyInput = e.KeyChar.ToString();
			if ((!Char.IsDigit(e.KeyChar))&&(e.KeyChar != '\b')){
				e.Handled = true;
			}
		}
		
		void TextLSBKeyPress(object sender, KeyPressEventArgs e){
			base.OnKeyPress(e);
			string keyInput = e.KeyChar.ToString();
			if ((!Char.IsDigit(e.KeyChar))&&(e.KeyChar != '\b')){
				e.Handled = true;
			}
		}
		
		void TextMSBTextChanged(object sender, EventArgs e){
			CalcBorder();
		}
		
		void TextLSBTextChanged(object sender, EventArgs e){
			CalcBorder();
		}
	}
}