using System;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;
using System.Threading;

namespace VHDLSimulator
{
	public partial class VHDLSimulator : Form
	{
		ProcessControl pCont;
		string appPath;

		public delegate void ControlChangeHandler (object Value,SimulationEventArgs timeInformation);

		public event ControlChangeHandler ControlChange;
		
		public VHDLSimulator(){
			InitializeComponent();

			appPath = Path.GetDirectoryName(Application.ExecutablePath);
			string newpath = System.Environment.GetEnvironmentVariable("PATH")+@"..\Ghd\lib;..\Ghdl\bin;"+appPath;
			System.Environment.SetEnvironmentVariable("PATH",newpath);

			pCont=new ProcessControl();
			pCont.SimulationChange +=new ProcessControl.SimulationChangeHandler(SimulatonChanged);
			
			this.Text="VHDL Simulator 1.0  (Author: Prof. Dr. Rdiger Heintz)";
		}
		
		public void SimulatonChanged(object sender, EventArgs  e){
			if(InvokeRequired)
				Invoke(new EventHandler(SimulatonChanged),new object[] {sender, e});
			else{
				string InD=((SimulationEventArgs)e).Value;
				LED_Out.setAllLEDs(InD);
				SevenSeg.SegmentStr=InD;
				bitProgressBar.SetValue(InD);
			}
		}
		
		void Changed(object sender, EventArgs e){
			string Result=LED_In.getLEDString();
			Result=clockgeneratorControl.insertString(Result);
			
			if (ControlChange != null) ControlChange(this,new SimulationEventArgs(Result));
		}
		
		string RunProcess(string exe, string args){
			string ResultText="";
			ProcessStartInfo pSInfo = new ProcessStartInfo(exe,args);
			pSInfo.WorkingDirectory=appPath;
			pSInfo.UseShellExecute = false;
			pSInfo.ErrorDialog = false;
			pSInfo.CreateNoWindow=true;
			pSInfo.RedirectStandardError = true;
			pSInfo.RedirectStandardInput = true;
			pSInfo.RedirectStandardOutput = true;
			Process process = new Process();
			process.StartInfo = pSInfo;
			bool processStarted = process.Start();
			
			process.WaitForExit();
			ResultText+=pSInfo.FileName+" "+pSInfo.Arguments+Environment.NewLine;
			string OutText=process.StandardOutput.ReadToEnd();
			if(OutText.Length>0) ResultText+= "Output" + Environment.NewLine + OutText +Environment.NewLine;
			string ErrorText=process.StandardError.ReadToEnd();
			if(ErrorText.Length>0) ResultText+=  "Error" + Environment.NewLine + ErrorText +Environment.NewLine;;
			return ResultText;
		}
		
		void CompileToolStripMenuItemClick(object sender, EventArgs e){
			pCont.StopProcess();
			string displayText="";
			
			foreach (string FilePath in Directory.GetFiles(appPath, "*.vhd")){
				if (Path.GetExtension(FilePath)!=".vhd") continue;
				
				displayText+=RunProcess("ghdl.exe", "-a \""+Path.GetFileName(FilePath)+"\"");
			}
			foreach (string FilePath in Directory.GetFiles(appPath, "*.vhdl")){
				displayText+=RunProcess("ghdl.exe", "-a \""+Path.GetFileName(FilePath)+"\"");
			}
			
			displayText+=RunProcess("ghdl.exe", "-e testbench");
			
			MessageBox.Show(displayText);
		}
		
		void StartToolStripMenuItemClick(object sender, EventArgs e){
			pCont.StartProcess(this);
			Changed(null,EventArgs.Empty);
		}
		
		void CleanToolStripMenuItemClick(object sender, EventArgs e){
			pCont.StopProcess();
			string displayText="";
			displayText+=RunProcess("ghdl.exe", "--clean");
			displayText+=RunProcess("ghdl.exe", "--remove");
			foreach (string FilePath in Directory.GetFiles(appPath, "*.*")){
				try {
					if(FilePath.ToLower().EndsWith(".vcd")||FilePath.ToLower().EndsWith(".pdb")){
						
						File.Delete(FilePath);
					}
				}
				catch (Exception ex){
					MessageBox.Show(ex.Message,"Error");
				}
			}
			MessageBox.Show(displayText);
		}
	}

	public class ProcessControl:Form
	{
		Process process;
		string ControlStatus="";
		public ProcessControl(){}
		
		public delegate void SimulationChangeHandler (object Value,SimulationEventArgs e);
		public event SimulationChangeHandler SimulationChange;
		
		public void StopProcess(){
			try{
			process.Kill();
			}
			catch{
			}			
		}


		public bool StartProcess(VHDLSimulator parent){
			try{
			process.Kill();
			}
			catch{
			}
			
			parent.ControlChange += new VHDLSimulator.ControlChangeHandler(ControlChanged);

			ProcessStartInfo processStartInfo = new ProcessStartInfo("ghdl.exe", "-r testbench --vcd=Result.vcd");
			processStartInfo.ErrorDialog = false;
			processStartInfo.UseShellExecute = false;
			processStartInfo.RedirectStandardError = true;
			processStartInfo.RedirectStandardInput = true;
			processStartInfo.RedirectStandardOutput = true;
			processStartInfo.CreateNoWindow=true;
			process = new Process();
			process.StartInfo = processStartInfo;
			bool processStarted = process.Start();
			if (processStarted==false) return false;

			Thread m_threadRec;
			try{
				m_threadRec = new Thread(new ThreadStart(ReadProcessOutput));
				m_threadRec.IsBackground=true;
				m_threadRec.Start();
				return true;
			}
			catch{
				m_threadRec = null;
				return false;
			}
		}
		
		public void ControlChanged(object sender, EventArgs  e){
			if(InvokeRequired)
				Invoke(new EventHandler(ControlChanged),new object[] {sender, e});
			else{
				ControlStatus=((SimulationEventArgs)e).Value;
			}
		}

		protected virtual void ReadProcessOutput()
		{
			string strLine;
			string strLineOld="";
			
			try{
				StreamReader outputReader = process.StandardOutput;
				StreamReader errorReader = process.StandardError;
				
				while ((strLine = process.StandardOutput.ReadLine()) != null){
					if(strLine!=""){
						
						process.StandardInput.WriteLine(ControlStatus);
						
						if (strLineOld!=strLine){
							strLineOld=strLine;
							
							if (SimulationChange != null)
								SimulationChange(this,new SimulationEventArgs(strLine));
						}
					}
					System.Threading.Thread.Sleep(10);
				}

				string Result="Out:\n"+process.StandardOutput.ReadToEnd()+
					"Error:\n"+process.StandardError.ReadToEnd();
				if(Result.Length>14)
					MessageBox.Show(Result);
				
				
				if(outputReader != null)outputReader.Close();
				if(errorReader != null)errorReader.Close();
				
			}
			catch (Exception e){MessageBox.Show(e.Message);}
		}
	}
	
	public class SimulationEventArgs : EventArgs
	{
		public SimulationEventArgs(string Val){
			this.Value = Val;
		}
		public readonly string Value;
	};
}